@@ -15,11 +15,11 @@
  * together.
  */
 
-/* 
+/*
  * Copyright (C) 2001, 2002, 2003 Red Hat, Inc.
  * Copyright (C) 2004, 2005 Elijah Newren
  * Copyright (C) 2009 Thomas Thurman
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -29,7 +29,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street - Suite 500, Boston, MA
@@ -122,9 +122,9 @@ meta_window_reload_properties_from_xwindow (MetaWindow *window,
 
   g_return_if_fail (properties != NULL);
   g_return_if_fail (n_properties > 0);
-  
+
   values = g_new0 (MetaPropValue, n_properties);
-  
+
   i = 0;
   while (i < n_properties)
     {
@@ -132,7 +132,7 @@ meta_window_reload_properties_from_xwindow (MetaWindow *window,
       init_prop_value (window, hooks, &values[i]);
       ++i;
     }
-  
+
   meta_prop_get_values (window->display, xwindow,
                         values, n_properties);
 
@@ -141,13 +141,13 @@ meta_window_reload_properties_from_xwindow (MetaWindow *window,
     {
       MetaWindowPropHooks *hooks = find_hooks (window->display, properties[i]);
       reload_prop_value (window, hooks, &values[i], initial);
-      
+
       ++i;
     }
 
   meta_prop_free_values (values, n_properties);
-  
-  g_free (values);
+
+  free (values);
 }
 
 LOCAL_SYMBOL void
@@ -192,7 +192,7 @@ meta_window_load_initial_properties (MetaWindow *window)
 
   meta_prop_free_values (values, n_properties);
 
-  g_free (values);
+  free (values);
 }
 
 /* Fill in the MetaPropValue used to get the value of "property" */
@@ -230,9 +230,9 @@ reload_wm_client_machine (MetaWindow    *window,
                           MetaPropValue *value,
                           gboolean       initial)
 {
-  g_free (window->wm_client_machine);
+  free (window->wm_client_machine);
   window->wm_client_machine = NULL;
-  
+
   if (value->type != META_PROP_VALUE_INVALID)
     window->wm_client_machine = g_strdup (value->v.str);
 
@@ -245,7 +245,7 @@ reload_theme_icon_name (MetaWindow    *window,
                         MetaPropValue *value,
                         gboolean       initial)
 {
-  g_free (window->theme_icon_name);
+  free (window->theme_icon_name);
   window->theme_icon_name = NULL;
 
   if (value->type != META_PROP_VALUE_INVALID)
@@ -265,7 +265,7 @@ reload_progress (MetaWindow    *window,
       if (window->progress != value->v.cardinal)
         {
           window->progress = value->v.cardinal;
-          g_object_notify (window, "progress");
+          g_object_notify ((GObject*) window, "progress");
 
           meta_topic (META_DEBUG_WINDOW_STATE,
                       "Read XAppGtkWindow progress prop %u for %s\n",
@@ -277,7 +277,7 @@ reload_progress (MetaWindow    *window,
       if (window->progress != 0)
         {
           window->progress = 0;
-          g_object_notify (window, "progress");
+          g_object_notify ((GObject*) window, "progress");
 
           meta_topic (META_DEBUG_WINDOW_STATE,
                       "Read XAppGtkWindow progress prop %u for %s\n",
@@ -298,7 +298,7 @@ reload_progress_pulse (MetaWindow    *window,
       if (window->progress_pulse != new_val)
         {
           window->progress_pulse = new_val;
-          g_object_notify (window, "progress-pulse");
+          g_object_notify ((GObject*) window, "progress-pulse");
 
           meta_topic (META_DEBUG_WINDOW_STATE,
                       "Read XAppGtkWindow progress-pulse prop %s for %s\n",
@@ -310,7 +310,7 @@ reload_progress_pulse (MetaWindow    *window,
       if (window->progress_pulse)
         {
           window->progress_pulse = FALSE;
-          g_object_notify (window, "progress-pulse");
+          g_object_notify ((GObject*) window,  "progress-pulse");
 
           meta_topic (META_DEBUG_WINDOW_STATE,
                       "Read XAppGtkWindow progress-pulse prop %s for %s\n",
@@ -340,10 +340,7 @@ static void
 reload_icon (MetaWindow    *window,
              Atom           atom)
 {
-  meta_icon_cache_property_changed (&window->icon_cache,
-                                    window->display,
-                                    atom);
-  meta_window_queue(window, META_QUEUE_UPDATE_ICON);
+  g_signal_emit_by_name (window, "icon-changed", window);
 }
 
 static void
@@ -445,7 +442,7 @@ reload_net_wm_pid (MetaWindow    *window,
   if (value->type != META_PROP_VALUE_INVALID)
     {
       gulong cardinal = (int) value->v.cardinal;
-      
+
       if (cardinal <= 0)
         meta_warning (_("Application set a bogus _NET_WM_PID %lu\n"),
                       cardinal);
@@ -477,6 +474,7 @@ reload_net_wm_user_time_window (MetaWindow    *window,
 {
   if (value->type != META_PROP_VALUE_INVALID)
     {
+      MetaWindow *prev_owner;
       /* Unregister old NET_WM_USER_TIME_WINDOW */
       if (window->user_time_window != None)
         {
@@ -489,6 +487,15 @@ reload_net_wm_user_time_window (MetaWindow    *window,
                         NoEventMask);
         }
 
+      /* Ensure the new user time window is not used on another MetaWindow,
+       * and unset its user time window if that is the case.
+       */
+      prev_owner = meta_display_lookup_x_window (window->display, value->v.xwindow);
+      if (prev_owner && prev_owner->user_time_window == value->v.xwindow)
+        {
+          meta_display_unregister_x_window (window->display, value->v.xwindow);
+          prev_owner->user_time_window = None;
+        }
 
       /* Obtain the new NET_WM_USER_TIME_WINDOW and register it */
       window->user_time_window = value->v.xwindow;
@@ -545,12 +552,12 @@ set_title_text (MetaWindow  *window,
 {
   char hostname[HOST_NAME_MAX + 1];
   gboolean modified = FALSE;
-  
+
   if (!target)
     return FALSE;
-  
-  g_free (*target);
-  
+
+  free (*target);
+
   if (!title)
     *target = g_strdup ("");
   else if (g_utf8_strlen (title, MAX_TITLE_LENGTH + 1) > MAX_TITLE_LENGTH)
@@ -594,7 +601,7 @@ set_window_title (MetaWindow *window,
                   const char *title)
 {
   char *str;
- 
+
   gboolean modified =
     set_title_text (window,
                     window->using_net_wm_visible_name,
@@ -602,12 +609,12 @@ set_window_title (MetaWindow *window,
                     window->display->atom__NET_WM_VISIBLE_NAME,
                     &window->title);
   window->using_net_wm_visible_name = modified;
-  
+
   /* strndup is a hack since GNU libc has broken %.10s */
   str = g_strndup (window->title, 10);
-  g_free (window->desc);
+  free (window->desc);
   window->desc = g_strdup_printf ("0x%lx (%s)", window->xwindow, str);
-  g_free (str);
+  free (str);
 
   if (window->frame)
     meta_ui_set_frame_title (window->screen->ui,
@@ -650,7 +657,7 @@ reload_wm_name (MetaWindow    *window,
                     value->v.str);
       return;
     }
-  
+
   if (value->type != META_PROP_VALUE_INVALID)
     {
       set_window_title (window, value->v.str);
@@ -664,6 +671,74 @@ reload_wm_name (MetaWindow    *window,
     }
 }
 
+static void
+meta_window_set_opaque_region (MetaWindow     *window,
+                               cairo_region_t *region)
+{
+  if (cairo_region_equal (window->opaque_region, region))
+    return;
+
+  g_clear_pointer (&window->opaque_region, cairo_region_destroy);
+
+  if (region != NULL)
+    window->opaque_region = cairo_region_reference (region);
+
+  meta_compositor_window_shape_changed (window->display->compositor, window);
+}
+
+static void
+reload_opaque_region (MetaWindow    *window,
+                      MetaPropValue *value,
+                      gboolean       initial)
+{
+  cairo_region_t *opaque_region = NULL;
+
+  if (value->type != META_PROP_VALUE_INVALID)
+    {
+      gulong *region = value->v.cardinal_list.cardinals;
+      int nitems = value->v.cardinal_list.n_cardinals;
+
+      cairo_rectangle_int_t *rects;
+      int i, rect_index, nrects;
+
+      if (nitems % 4 != 0)
+        {
+          meta_verbose ("_NET_WM_OPAQUE_REGION does not have a list of 4-tuples.");
+          goto out;
+        }
+
+      /* empty region */
+      if (nitems == 0)
+        goto out;
+
+      nrects = nitems / 4;
+
+      rects = g_new (cairo_rectangle_int_t, nrects);
+
+      rect_index = 0;
+      i = 0;
+      while (i < nitems)
+        {
+          cairo_rectangle_int_t *rect = &rects[rect_index];
+
+          rect->x = region[i++];
+          rect->y = region[i++];
+          rect->width = region[i++];
+          rect->height = region[i++];
+
+          rect_index++;
+        }
+
+      opaque_region = cairo_region_create_rectangles (rects, nrects);
+
+      free (rects);
+    }
+
+ out:
+  meta_window_set_opaque_region (window, opaque_region);
+  cairo_region_destroy (opaque_region);
+}
+
 static void
 reload_muffin_hints (MetaWindow    *window,
                      MetaPropValue *value,
@@ -688,7 +763,7 @@ reload_muffin_hints (MetaWindow    *window,
 
       if (changed)
         {
-          g_free (old_hints);
+          free (old_hints);
 
           if (new_hints)
             window->muffin_hints = g_strdup (new_hints);
@@ -700,7 +775,7 @@ reload_muffin_hints (MetaWindow    *window,
     }
   else if (window->muffin_hints)
     {
-      g_free (window->muffin_hints);
+      free (window->muffin_hints);
       window->muffin_hints = NULL;
 
       g_object_notify (G_OBJECT (window), "muffin-hints");
@@ -753,11 +828,11 @@ reload_wm_icon_name (MetaWindow    *window,
                     value->v.str);
       return;
     }
-  
+
   if (value->type != META_PROP_VALUE_INVALID)
     {
       set_icon_title (window, value->v.str);
-      
+
       meta_verbose ("Using WM_ICON_NAME for new title of %s: \"%s\"\n",
                     window->desc, window->title);
     }
@@ -955,7 +1030,7 @@ reload_mwm_hints (MetaWindow    *window,
     meta_verbose ("Functions flag unset\n");
 
   meta_window_recalc_features (window);
-  
+
   /* We do all this anyhow at the end of meta_window_new() */
   if (!window->constructing)
     {
@@ -963,7 +1038,7 @@ reload_mwm_hints (MetaWindow    *window,
         meta_window_ensure_frame (window);
       else
         meta_window_destroy_frame (window);
-      
+
       meta_window_queue (window,
                          META_QUEUE_MOVE_RESIZE |
                          /* because ensure/destroy frame may unmap: */
@@ -980,15 +1055,15 @@ reload_wm_class (MetaWindow    *window,
                  gboolean       initial)
 {
   if (window->res_class)
-    g_free (window->res_class);
+    free (window->res_class);
   if (window->res_name)
-    g_free (window->res_name);
+    free (window->res_name);
 
   window->res_class = NULL;
   window->res_name = NULL;
 
   if (value->type != META_PROP_VALUE_INVALID)
-    { 
+    {
       if (value->v.class_hint.res_name)
         window->res_name = g_strdup (value->v.class_hint.res_name);
 
@@ -1026,32 +1101,32 @@ reload_net_startup_id (MetaWindow    *window,
 {
   guint32 timestamp = window->net_wm_user_time;
   MetaWorkspace *workspace = NULL;
-  
-  g_free (window->startup_id);
-  
+
+  free (window->startup_id);
+
   if (value->type != META_PROP_VALUE_INVALID)
     window->startup_id = g_strdup (value->v.str);
   else
     window->startup_id = NULL;
-    
+
   /* Update timestamp and workspace on a running window */
   if (!window->constructing)
   {
-    window->initial_timestamp_set = 0;  
+    window->initial_timestamp_set = 0;
     window->initial_workspace_set = 0;
-    
+
     if (meta_screen_apply_startup_properties (window->screen, window))
       {
-  
+
         if (window->initial_timestamp_set)
           timestamp = window->initial_timestamp;
         if (window->initial_workspace_set)
           workspace = meta_screen_get_workspace_by_index (window->screen, window->initial_workspace);
-    
+
         meta_window_activate_with_workspace (window, timestamp, workspace);
       }
   }
-  
+
   meta_verbose ("New _NET_STARTUP_ID \"%s\" for %s\n",
                 window->startup_id ? window->startup_id : "unset",
                 window->desc);
@@ -1151,7 +1226,7 @@ spew_size_hints_differences (const XSizeHints *old,
   if (FLAG_CHANGED (old, new, PWinGravity))
     meta_topic (META_DEBUG_GEOMETRY, "XSizeHints: PWinGravity now %s  (%d -> %d)\n",
                 FLAG_TOGGLED_ON (old, new, PWinGravity) ? "set" : "unset",
-                old->win_gravity, new->win_gravity);  
+                old->win_gravity, new->win_gravity);
 }
 
 LOCAL_SYMBOL void
@@ -1508,15 +1583,15 @@ reload_normal_hints (MetaWindow    *window,
   if (value->type != META_PROP_VALUE_INVALID)
     {
       XSizeHints old_hints;
-  
+
       meta_topic (META_DEBUG_GEOMETRY, "Updating WM_NORMAL_HINTS for %s\n", window->desc);
 
       old_hints = window->size_hints;
-  
+
       meta_set_normal_hints (window, value->v.size_hints.hints);
-      
+
       spew_size_hints_differences (&old_hints, &window->size_hints);
-      
+
       meta_window_recalc_features (window);
 
       if (!initial)
@@ -1530,12 +1605,12 @@ reload_wm_protocols (MetaWindow    *window,
                      gboolean       initial)
 {
   int i;
-  
+
   window->take_focus = FALSE;
   window->delete_window = FALSE;
   window->net_wm_ping = FALSE;
-  
-  if (value->type == META_PROP_VALUE_INVALID)    
+
+  if (value->type == META_PROP_VALUE_INVALID)
     return;
 
   i = 0;
@@ -1552,7 +1627,7 @@ reload_wm_protocols (MetaWindow    *window,
         window->net_wm_ping = TRUE;
       ++i;
     }
-  
+
   meta_verbose ("New _NET_STARTUP_ID \"%s\" for %s\n",
                 window->startup_id ? window->startup_id : "unset",
                 window->desc);
@@ -1568,7 +1643,7 @@ reload_wm_hints (MetaWindow    *window,
 
   old_group_leader = window->xgroup_leader;
   old_urgent = window->wm_hints_urgent;
-  
+
   /* Fill in defaults */
   window->input = TRUE;
   window->initially_iconic = FALSE;
@@ -1580,7 +1655,7 @@ reload_wm_hints (MetaWindow    *window,
   if (value->type != META_PROP_VALUE_INVALID)
     {
       const XWMHints *hints = value->v.wm_hints;
-      
+
       if (hints->flags & InputHint)
         window->input = hints->input;
 
@@ -1610,7 +1685,7 @@ reload_wm_hints (MetaWindow    *window,
     {
       meta_verbose ("Window %s changed its group leader to 0x%lx\n",
                     window->desc, window->xgroup_leader);
-      
+
       meta_window_group_leader_changed (window);
     }
 
@@ -1633,7 +1708,40 @@ reload_wm_hints (MetaWindow    *window,
                                     window->display,
                                     XA_WM_HINTS);
 
-  meta_window_queue (window, META_QUEUE_UPDATE_ICON | META_QUEUE_MOVE_RESIZE);
+  meta_window_queue (window, META_QUEUE_MOVE_RESIZE);
+}
+
+static void
+reload_gtk_hide_titlebar_when_maximized (MetaWindow    *window,
+                                         MetaPropValue *value,
+                                         gboolean       initial)
+{
+  gboolean requested_value = FALSE;
+  gboolean current_value = window->hide_titlebar_when_maximized;
+
+  if (meta_prefs_get_ignore_hide_titlebar_when_maximized ())
+    {
+      return;
+    }
+
+  if (value->type != META_PROP_VALUE_INVALID)
+    {
+      requested_value = ((int) value->v.cardinal == 1);
+      meta_verbose ("Request to hide titlebar for window %s.\n", window->desc);
+    }
+
+  if (requested_value == current_value)
+    return;
+
+  window->hide_titlebar_when_maximized = requested_value;
+
+  if (META_WINDOW_MAXIMIZED (window))
+    {
+      meta_window_queue (window, META_QUEUE_MOVE_RESIZE);
+
+      if (window->frame)
+        meta_ui_update_frame_style (window->screen->ui, window->frame->xwindow);
+    }
 }
 
 typedef struct
@@ -1685,7 +1793,7 @@ lookup_parent_by_client_leader (MetaWindow *transient,
       parent = data->window;
     }
 
-    g_free (data);
+    free (data);
 
     return parent;
 }
@@ -1826,7 +1934,7 @@ reload_gtk_theme_variant (MetaWindow    *window,
 
   if (g_strcmp0 (requested_variant, current_variant) != 0)
     {
-      g_free (current_variant);
+      free (current_variant);
 
       window->gtk_theme_variant = g_strdup (requested_variant);
 
@@ -1867,7 +1975,7 @@ reload_bypass_compositor (MetaWindow    *window,
                        MetaPropValue *value,        \
                        gboolean       initial)      \
   {                                                 \
-    g_free (window->var_name);                      \
+    free (window->var_name);                      \
                                                     \
     if (value->type != META_PROP_VALUE_INVALID)     \
       window->var_name = g_strdup (value->v.str);   \
@@ -1926,6 +2034,7 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
     { display->atom__NET_WM_PID,       META_PROP_VALUE_CARDINAL, reload_net_wm_pid,        TRUE,  TRUE },
     { XA_WM_NAME,                      META_PROP_VALUE_TEXT_PROPERTY, reload_wm_name,      TRUE,  TRUE },
     { display->atom__MUFFIN_HINTS,     META_PROP_VALUE_TEXT_PROPERTY, reload_muffin_hints, TRUE,  TRUE },
+    { display->atom__NET_WM_OPAQUE_REGION, META_PROP_VALUE_CARDINAL_LIST, reload_opaque_region, TRUE, TRUE },
     { display->atom__NET_WM_ICON_NAME, META_PROP_VALUE_UTF8,     reload_net_wm_icon_name,  TRUE,  FALSE },
     { XA_WM_ICON_NAME,                 META_PROP_VALUE_TEXT_PROPERTY, reload_wm_icon_name, TRUE,  FALSE },
     { display->atom__NET_WM_DESKTOP,   META_PROP_VALUE_CARDINAL, reload_net_wm_desktop,    TRUE,  FALSE },
@@ -1938,12 +2047,13 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
     { display->atom__NET_WM_STATE,     META_PROP_VALUE_ATOM_LIST, reload_net_wm_state,     TRUE,  FALSE },
     { display->atom__MOTIF_WM_HINTS,   META_PROP_VALUE_MOTIF_HINTS, reload_mwm_hints,      TRUE,  FALSE },
     { XA_WM_TRANSIENT_FOR,             META_PROP_VALUE_WINDOW,    reload_transient_for,    TRUE,  FALSE },
-    { display->atom__GTK_THEME_VARIANT, META_PROP_VALUE_UTF8,     reload_gtk_theme_variant, TRUE, FALSE },    
+    { display->atom__GTK_THEME_VARIANT, META_PROP_VALUE_UTF8,     reload_gtk_theme_variant, TRUE, FALSE },
+    { display->atom__GTK_HIDE_TITLEBAR_WHEN_MAXIMIZED, META_PROP_VALUE_CARDINAL,     reload_gtk_hide_titlebar_when_maximized, TRUE, FALSE },
     { display->atom__GTK_APPLICATION_ID,               META_PROP_VALUE_UTF8,         reload_gtk_application_id,               TRUE, FALSE },
     { display->atom__GTK_UNIQUE_BUS_NAME,              META_PROP_VALUE_UTF8,         reload_gtk_unique_bus_name,              TRUE, FALSE },
     { display->atom__GTK_APPLICATION_OBJECT_PATH,      META_PROP_VALUE_UTF8,         reload_gtk_application_object_path,      TRUE, FALSE },
-    { display->atom__GTK_WINDOW_OBJECT_PATH,           META_PROP_VALUE_UTF8,         reload_gtk_window_object_path,           TRUE, FALSE },    
-    { display->atom__GTK_APP_MENU_OBJECT_PATH,         META_PROP_VALUE_UTF8,         reload_gtk_app_menu_object_path,         TRUE, FALSE },    
+    { display->atom__GTK_WINDOW_OBJECT_PATH,           META_PROP_VALUE_UTF8,         reload_gtk_window_object_path,           TRUE, FALSE },
+    { display->atom__GTK_APP_MENU_OBJECT_PATH,         META_PROP_VALUE_UTF8,         reload_gtk_app_menu_object_path,         TRUE, FALSE },
     { display->atom__GTK_MENUBAR_OBJECT_PATH,          META_PROP_VALUE_UTF8,         reload_gtk_menubar_object_path,          TRUE, FALSE },
     { display->atom__GTK_FRAME_EXTENTS,                META_PROP_VALUE_CARDINAL_LIST,reload_gtk_frame_extents,                TRUE, FALSE },
     { display->atom__NET_WM_USER_TIME_WINDOW, META_PROP_VALUE_WINDOW, reload_net_wm_user_time_window, TRUE, FALSE },
@@ -1966,7 +2076,7 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
 
   MetaWindowPropHooks *table = g_memdup (hooks, sizeof (hooks)),
     *cursor = table;
-  
+
   g_assert (display->prop_hooks == NULL);
 
   display->prop_hooks_table = (gpointer) table;
@@ -1995,7 +2105,7 @@ meta_display_free_window_prop_hooks (MetaDisplay *display)
   g_hash_table_unref (display->prop_hooks);
   display->prop_hooks = NULL;
 
-  g_free (display->prop_hooks_table);
+  free (display->prop_hooks_table);
   display->prop_hooks_table = NULL;
 }
 
